package com.persagy.adm.diagram.core.model.base;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.google.gson.annotations.Expose;

/**
 * 布局和绘制组件的基类
 * @author zhaoyk
 */
abstract public class AbstractComponent implements IComponent {

	@Expose //使用gson保存文件
	protected String id;

	@Expose
	protected String name;

	@Expose
	protected String compType;

	@JsonIgnore //使用jackson输出到web端
	protected IContainer parent;

	@Expose
	protected PositionData position;

	protected XY location;

	protected XY size;

//	@Expose
//	private HashMap<String, Object> datas;

	protected AbstractComponent(){

	}

	@Override
	public String getId() {
		return id;
	}

	@Override
	public void setId(String id) {
		this.id = id;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public String getCompType() {
		return compType;
	}

	@Override
	public IContainer getParent() {
		return parent;
	}

	@Override
	public void setParent(IContainer parent) {
		this.parent = parent;
	}

	@Override
	public PositionData getPosition() {
		return position;
	}

	@Override
	public void setPosition(PositionData position) {
		this.position = position;
	}

	@Override
	public PositionData defaultPosition(){
		return null;
	}

	@Override
	public boolean isAbsolutePosition(){
		return position != null && position.isAbsolute();
	}

	@Override
	public XY getLocation() {
		return location;
	}

	@Override
	public void setLocation(XY location) {
		this.location = location;
	}

	@Override
	public XY getSize() {
		return size;
	}

	public void setSize(XY size) {
		this.size = size;
	}

//	@Override
//	public Object getData(String name){
//		return datas != null ? datas.get(name) : null;
//	}
//
//	@Override
//	public void setData(String name, Object value){
//		if(datas == null)
//			datas = new HashMap<>();
//		datas.put(name, value);
//	}

	/**
	 * 相对根容器的坐标位置
	 */
	public XY locationToRoot(){
		XY p = new XY(0, 0);
		IComponent c = this;
		while (c != null){
			XY pos = c.getLocation();
			if(pos == null)
				return null;

			p.x += pos.x;
			p.y += pos.y;

			c = c.getParent();
		}
		return p;
	}

}
