package com.persagy.adm.diagram.core.util;

import com.google.gson.*;
import com.persagy.adm.diagram.core.model.base.Container;
import com.persagy.adm.diagram.core.model.base.IComponent;
import org.springframework.stereotype.Component;

import java.lang.reflect.Type;

/**
 * gson解析和输出工具
 * @author zhaoyk
 */
@Component
public class GsonUtil {

	private Gson gson;

	private Gson formattedGson;

	public GsonUtil(){
		GsonBuilder gsonBuilder = new GsonBuilder().excludeFieldsWithoutExposeAnnotation();
		gsonBuilder.registerTypeAdapter(IComponent.class, new CompAdapter());
		gson = gsonBuilder.create();

		formattedGson = new GsonBuilder().setPrettyPrinting().create();
	}

	class CompAdapter implements JsonDeserializer<IComponent>, JsonSerializer<IComponent> {

		@Override
		public IComponent deserialize(JsonElement jsonElement, Type type, JsonDeserializationContext jsonDeserializationContext) throws JsonParseException {
			Class<? extends IComponent> target = Container.class;
			if(Container.TYPE.equals(jsonElement.getAsJsonObject().get("compType")))
				target = Container.class;
			else {
				//TODO 其他类型
			}
			return gson.fromJson(jsonElement, target);
		}

		@Override
		public JsonElement serialize(IComponent iComponent, Type type, JsonSerializationContext jsonSerializationContext) {
			return gson.toJsonTree(iComponent, iComponent.getClass());
		}

	}

	public Gson gson(){
		return gson;
	}

	public Gson formattedGson(){
		return formattedGson;
	}

}
